var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as path from 'path';
import * as fs from 'fs';
import { globbySync } from 'globby';
import getPort from 'get-port';
import colors from 'colors';
const killPort = require('kill-port');
export function getJSONFile(relPath) {
    return require(path.join(process.cwd(), relPath));
}
export function setDefaultServeSettings(settings) {
    const defaultServeSettings = {
        fullScreenErrors: true,
        loggingLevel: 'normal',
        hotRefresh: false,
        openUrl: undefined
    };
    settings.serve = settings.serve || {};
    settings.serve = Object.assign(defaultServeSettings, settings.serve);
    if (settings.cli.isLibraryComponent) {
        settings.serve.openUrl = undefined;
    }
}
export function getLoggingLevel(level) {
    if (level === 'minimal') {
        return {
            all: false,
            colors: true,
            errors: true
        };
    }
    if (level === 'normal') {
        return {
            all: false,
            colors: true,
            errors: true,
            timings: true,
            entrypoints: true
        };
    }
    if (level === 'detailed') {
        return {
            all: false,
            colors: true,
            errors: true,
            timings: true,
            assets: true,
            warnings: true
        };
    }
    throw new Error('Unsupported log level: ' + level);
}
export function getEntryPoints(entry) {
    const newEntry = {};
    for (const key in entry) {
        let entryPath = entry[key];
        if (entryPath.indexOf('bundle-entries') === -1) {
            entryPath = createTsEntryPath(entryPath);
        }
        else {
            let bundleContent = fs.readFileSync(entryPath).toString();
            bundleContent = createTsEntriesForBundledPackage(bundleContent);
            fs.writeFileSync(entryPath, bundleContent);
        }
        newEntry[key] = entryPath;
    }
    return newEntry;
}
function createTsEntriesForBundledPackage(content) {
    const search = /require\('(?<jsPath>.*)'\)/gi;
    let newContent = content.slice();
    let match = search.exec(content);
    do {
        const jsPath = match.groups.jsPath;
        const tsPath = createTsEntryPath(jsPath);
        newContent = newContent.replace(jsPath, tsPath);
    } while ((match = search.exec(content)) !== null);
    return newContent;
}
function createTsEntryPath(jsPath) {
    const libSearchRegexp1 = /\/lib\//gi;
    const libSearchRegexp2 = /\\lib\\/gi;
    const srcPathToReplace1 = '/src/';
    const srcPathToReplace2 = '\\src\\';
    const tsPath = jsPath
        .replace(libSearchRegexp1, srcPathToReplace1)
        .replace(libSearchRegexp2, srcPathToReplace2)
        .slice(0, -3) + '.ts';
    return getEntryPath(tsPath);
}
function getEntryPath(tsPath) {
    if (fs.existsSync(tsPath)) {
        return tsPath;
    }
    tsPath = tsPath + 'x';
    if (fs.existsSync(tsPath)) {
        return tsPath;
    }
    throw new Error('Unable to resolve entry path. Path received: ' + tsPath);
}
export function addCopyLocalizedResources(localizedResources) {
    const patterns = [];
    for (const resourceKey in localizedResources) {
        const resourcePath = localizedResources[resourceKey];
        const from = resourcePath.replace(/^lib/gi, 'src').replace('{locale}', '*');
        patterns.push({
            flatten: true,
            from,
            noErrorOnMissing: true,
            to: (data) => {
                let fileName = path.basename(data.absoluteFilename);
                if (!resourcePath.endsWith('{locale}.js')) {
                    fileName = 'default.js';
                }
                return resourceKey + '_' + fileName;
            }
        });
    }
    return patterns;
}
export function trimLeft(str, charlist) {
    return str.replace(new RegExp('^[' + charlist + ']+'), '');
}
export function trimRight(str, charlist) {
    return str.replace(new RegExp('[' + charlist + ']+$'), '');
}
export function trim(str, charlist) {
    return trimRight(trimLeft(str, charlist), charlist);
}
export function createKeyFromPath(path) {
    return trimLeft(path, '/\\\\').replace(/\//gi, '|').replace(/\\/gi, '|');
}
export function createResourcesMap(localizedResources) {
    const resourcesMap = {};
    for (const resourceKey in localizedResources) {
        const resourcePath = localizedResources[resourceKey];
        const search = resourcePath.replace(/^lib/gi, 'src').replace('{locale}.js', '*.ts');
        const exclude = '!' + search.replace('*.ts', '*.d.ts');
        const typescriptResources = globbySync([search, exclude], {
            cwd: process.cwd()
        });
        if (!(typescriptResources === null || typescriptResources === void 0 ? void 0 : typescriptResources.length)) {
            continue;
        }
        for (const resourcePath of typescriptResources) {
            const key = createKeyFromPath(resourcePath);
            const fileName = `${resourceKey}_${path.basename(resourcePath).replace('.ts', '.js')}`;
            resourcesMap[key] = {
                path: trimLeft(resourcePath, '/\\\\'),
                fileName
            };
        }
    }
    return resourcesMap;
}
export function freePortIfInUse(port) {
    return __awaiter(this, void 0, void 0, function* () {
        const freePort = yield getPort({ port, host: 'localhost' });
        if (freePort !== port) {
            console.log(colors.yellow(`The port ${port} is in use. Trying to release...`));
            yield killPort(port);
            console.log(colors.yellow(`The port ${port} is successfully released.`));
        }
    });
}
export function checkVersions() {
    const packageJson = getJSONFile('package.json');
    const spfxVersion = getMinorVersion(packageJson, '@microsoft/sp-build-web');
    const fastServeVersion = getMinorVersion(packageJson, 'spfx-fast-serve-helpers');
    if (spfxVersion !== fastServeVersion) {
        throw new Error(`SPFx Fast Serve: version mismatch. We detected the usage of SPFx 1.${spfxVersion}, but "spfx-fast-serve-helpers" version is 1.${fastServeVersion}. Please change "spfx-fast-serve-helpers" version to ~1.${spfxVersion}.0, delete node_modules, package-lock.json and reinstall dependencies.`);
    }
}
function getMinorVersion(packageJson, dependecyToCheck) {
    let version = packageJson.devDependencies[dependecyToCheck];
    if (!version) {
        version = packageJson.dependencies[dependecyToCheck];
    }
    if (!version) {
        throw new Error(`SPFx Fast Serve: unable to find dependency ${dependecyToCheck}`);
    }
    if (version.indexOf('~') === 0 || version.indexOf('^') === 0) {
        version = version.substr(1);
    }
    return parseInt(version.split('.')[1]);
}
export function getExternalComponents() {
    const originalWebpackConfig = getJSONFile('temp/_webpack_config.json');
    for (const plugin of originalWebpackConfig.plugins) {
        if (plugin._options && plugin._options.externalComponents) {
            return plugin._options.externalComponents;
        }
    }
    throw new Error('Unable to resolve AsyncComponentPlugin');
}
export function createLocalExternals(externals) {
    if (!externals)
        return null;
    const result = {};
    for (const name in externals) {
        const info = externals[name];
        if (typeof info === 'string') {
            if (info.startsWith('http')) {
                continue;
            }
            result[name] = { path: info };
        }
        else {
            if (info.path.startsWith('http')) {
                continue;
            }
            result[name] = { path: info.path };
        }
    }
    return result;
}
export function addCopyLocalExternals(externals, manifest, originalEntries) {
    if (!externals)
        return [];
    const patterns = [];
    for (const jsModule of manifest) {
        if (jsModule.loaderConfig
            && jsModule.loaderConfig.entryModuleId
            && originalEntries.indexOf(jsModule.loaderConfig.entryModuleId) !== -1) {
            for (const resourceKey in jsModule.loaderConfig.scriptResources) {
                const resource = jsModule.loaderConfig.scriptResources[resourceKey];
                if (externals[resourceKey]) {
                    const from = externals[resourceKey].path;
                    const to = resource.path;
                    if (!hasPattern(patterns, to)) {
                        patterns.push({
                            flatten: true,
                            from,
                            noErrorOnMissing: true,
                            to
                        });
                    }
                }
            }
        }
    }
    return patterns;
}
function hasPattern(patterns, to) {
    for (const pattern of patterns) {
        if (pattern.to === to) {
            return true;
        }
    }
    return false;
}
//# sourceMappingURL=helpers.js.map